package main

import (
	"fmt"
	"os"
	"path/filepath"
)

import "gitlab.com/beoran/muesli"
import "github.com/peterh/liner"

func runLine(vm *muesli.VM, in string) error {	
	parser := muesli.NewParserFromString(in)
	ast := parser.Parse()
	err := ast.ToError()
	if err != nil {
		return err
	}
	result := vm.RunAst(*ast, muesli.NewListValue())
	if result != nil { 
		for _, val := range result { 
			if val != nil { 
				fmt.Printf(">>%s\n", val.String())
			} else {
				fmt.Printf(">>nil\n")
			}
		}
	}
	
	return nil
}

func runLines(vm *muesli.VM, line *liner.State) error {
	for { 	
		if in, err := line.Prompt("> "); err == nil {
			err = runLine(vm, in)
			if err != nil { 
				fmt.Print("Error %s: \n", err)
			}
			line.AppendHistory(in)
		} else if err == liner.ErrPromptAborted {
			fmt.Print("Aborted\n")
			return nil
		} else {
			fmt.Print("Error reading line: %s\n", err)
		}
	}
	return nil
}

func main() {
	vm := muesli.NewVM()
	// vm.Tracer = &muesli.FmtTracer{}
	vm.RegisterBuiltins() 
	line := liner.NewLiner()
	defer line.Close()

	line.SetCtrlCAborts(true)
	home, _ := os.UserHomeDir()	
	historyName := filepath.Join(home, ".muesli_history")

	/* line.SetCompleter(func(line string) (c []string) {
		for _, n := range names {
			if strings.HasPrefix(n, strings.ToLower(line)) {
				c = append(c, n)
			}
		}
		return
	})
	*/

	if f, err := os.Open(historyName); err == nil {
		line.ReadHistory(f)
		f.Close()
	} 
	
	runLines(vm, line)
	
	if f, err := os.Create(historyName); err != nil {
		fmt.Print("Error writing history file: %s\n", err)
	} else {
		line.WriteHistory(f)
		f.Close()
	}
}


